@@ -58,11 +58,6 @@
 #include "psycopg/adapter_datetime.h"
 HIDDEN PyObject *pyDateTimeModuleP = NULL;
 
-/* pointers to the psycopg.tz classes */
-HIDDEN PyObject *pyPsycopgTzModule = NULL;
-HIDDEN PyObject *pyPsycopgTzLOCAL = NULL;
-HIDDEN PyObject *pyPsycopgTzFixedOffsetTimezone = NULL;
-
 HIDDEN PyObject *psycoEncodings = NULL;
 
 #ifdef PSYCOPG_DEBUG
@@ -630,8 +625,10 @@ psyco_GetDecimalType(void)
 static PyObject *
 psyco_make_description_type(void)
 {
-    PyObject *nt = NULL;
     PyObject *coll = NULL;
+    PyObject *nt = NULL;
+    PyTypeObject *t = NULL;
+    PyObject *s = NULL;
     PyObject *rv = NULL;
 
     /* Try to import collections.namedtuple */
@@ -645,12 +642,26 @@ psyco_make_description_type(void)
     }
 
     /* Build the namedtuple */
-    rv = PyObject_CallFunction(nt, "ss", "Column",
-        "name type_code display_size internal_size precision scale null_ok");
+    if(!(t = (PyTypeObject *)PyObject_CallFunction(nt, "ss", "Column",
+        "name type_code display_size internal_size precision scale null_ok"))) {
+        goto exit;
+    }
+
+    /* Export the tuple on the extensions module
+     * Required to guarantee picklability on Py > 3.3 (see Python issue 21374)
+     * for previous Py version the module is psycopg2 anyway but for consistency
+     * we'd rather expose it from the extensions module. */
+    if (!(s = Text_FromUTF8("psycopg2.extensions"))) { goto exit; }
+    if (0 > PyDict_SetItemString(t->tp_dict, "__module__", s)) { goto exit; }
+
+    rv = (PyObject *)t;
+    t = NULL;
 
 exit:
     Py_XDECREF(coll);
     Py_XDECREF(nt);
+    Py_XDECREF((PyObject *)t);
+    Py_XDECREF(s);
 
     return rv;
 
@@ -859,18 +870,6 @@ INIT_MODULE(_psycopg)(void)
     Py_TYPE(&pydatetimeType) = &PyType_Type;
     if (PyType_Ready(&pydatetimeType) == -1) goto exit;
 
-    /* import psycopg2.tz anyway (TODO: replace with C-level module?) */
-    pyPsycopgTzModule = PyImport_ImportModule("psycopg2.tz");
-    if (pyPsycopgTzModule == NULL) {
-        Dprintf("initpsycopg: can't import psycopg2.tz module");
-        PyErr_SetString(PyExc_ImportError, "can't import psycopg2.tz module");
-        goto exit;
-    }
-    pyPsycopgTzLOCAL =
-        PyObject_GetAttrString(pyPsycopgTzModule, "LOCAL");
-    pyPsycopgTzFixedOffsetTimezone =
-        PyObject_GetAttrString(pyPsycopgTzModule, "FixedOffsetTimezone");
-
     /* initialize the module and grab module's dictionary */
 #if PY_MAJOR_VERSION < 3
     module = Py_InitModule("_psycopg", psycopgMethods);
@@ -912,6 +911,7 @@ INIT_MODULE(_psycopg)(void)
     PyModule_AddObject(module, "Notify", (PyObject*)&notifyType);
     PyModule_AddObject(module, "Xid", (PyObject*)&xidType);
     PyModule_AddObject(module, "Diagnostics", (PyObject*)&diagnosticsType);
+    PyModule_AddObject(module, "Column", psyco_DescriptionType);
 #ifdef PSYCOPG_EXTENSIONS
     PyModule_AddObject(module, "lobject", (PyObject*)&lobjectType);
 #endif
